//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Vektorgraphic on M16C62 Processor
// Shows a rotating spaceship flying around a planet
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// (c) 2.2002   Holger Buss  Germany
// mailto:buss@mikrocontroller.com
// http://www.mikrocontroller.com
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the LCD (240*64) with T6963-controller is connected as:
// Look into t6963.h
//
// Ported to Atmel ATMega323 by Holger Klabunde
// I made very small changes in rotation routines to remove
// compiler errors and warnings. Replaced original line drawing
// function because i had some errors on the display.
// Low level hardware routines replaced by routines in t6963.c
//
// ATMega CPU clock 8MHz
// Compiler: AVR-GCC 3.2
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include <stdlib.h>
#include "main.h"
#include "t6963.h"
#include "protos.h"

#define     MUL 256      // because of fixcomma-calculations

unsigned char Text[40]="H.Buss  \0";

//Drawing functions from original programm replaced
//by functions in T6963.c and Draw.c. Tell the compiler
//new function names, so we don't have to change
//original code below !
#define cput(a)	WriteCommand((a))
#define dput(a)	WriteData((a))
#define lcd_setpixel(a,b) SetPixel((a),(b),1)
#define Cls() ClearScreen()
#define LCD_Line(x,y,x1,y1) DrawLine((x),(y),(x1),(y1))
#define LCD_Text(a,b,c) LCDText((c),(b),(a))

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// To calculate sin and cosin (Tables are much faster than the library)
// 62 points because of 2 Pi = 6,2.. 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
const int SIN[64]={
0,25,50,75,99,122,144,164,183,200,215,228,238,246,252,255,255,253,249,242,232,
220,206,190,172,153,131,109,85,61,36,10,-14,-40,-65,-89,-113,-135,-156,-176,
-193,-209,-223,-234,-243,-250,-254,-255,-255,-251,-245,-237,-226,-213,-197,
-180,-161,-140,-118,-95,-71,-46,-21};

const int COS[64]={ 255,254,250,244,235,224,211,195,178,159,138,116,92,68,43,18,-7,-32,
-58,-82,-106,-129,-150,-170,-188,-205,-219,-231,-241,-248,-253,-255,-255,-252,
-247,-239,-229,-217,-202,-185,-167,-147,-125,-102,-78,-53,-28,
-3,22,47,72,96,119,141,162,181,198,213,226,237,245,251,255};
//                     ^^^ was 16 ! Inserted right value HK

// Spaceship  (like Viper in C64-Elite)
#define  PNT_A    00,+00,+15  // defines the 3D vektors (edges of the spaceship)
#define  PNT_B   -70,+00,-15  // Nose
#define  PNT_C   +10,+40,-15
#define  PNT_F   +10,-40,-15
#define  PNT_D   +35,+20,-15
#define  PNT_E   +35,-20,-15
#define  PNT_G   -40,+00,-15
#define  PNT_H   +10,+20,-15
#define  PNT_I   +10,-20,-15
#define  CNT_UFO  13  // consists of 13 lines
const int UFO[CNT_UFO][6]=
       {
		{ PNT_B,PNT_C},	 // Lines between these Points
		{ PNT_B,PNT_A},
		{ PNT_B,PNT_F},
		{ PNT_C,PNT_A},
		{ PNT_F,PNT_A},
		{ PNT_D,PNT_A},
		{ PNT_E,PNT_A},
		{ PNT_E,PNT_F},
		{ PNT_E,PNT_D},
		{ PNT_C,PNT_D},
		{ PNT_G,PNT_H},
		{ PNT_G,PNT_I},
		{ PNT_H,PNT_I}
	   };

// Pyramide
#define PYR_A    00,+00,-10
#define PYR_B   -15,-8,+10
#define PYR_C   +15,-8,+10
#define PYR_D   +00,17,+10
#define CNT_PYR  6
const int PYR[CNT_PYR][6]=
//              ^^^ Was CNT_UFO. Too large ;)
       {
		{ PYR_A,PYR_B},
		{ PYR_A,PYR_D},
		{ PYR_A,PYR_C},
		{ PYR_B,PYR_C},
		{ PYR_B,PYR_D},
		{ PYR_D,PYR_C}
	   };

// Quader added by Holger Klabunde
#define Q_A    -10,-10,10
#define Q_B   10,-10,10
#define Q_C   10,-10,-10
#define Q_D   -10,-10,-10
#define Q_E   -10,10,10
#define Q_F   10,10,10
#define Q_G   10,10,-10
#define Q_H   -10,10,-10
#define CNT_Q 12
const int Q[CNT_Q][6]=
       {
		{ Q_A,Q_B},
		{ Q_A,Q_D},
		{ Q_A,Q_E},
		{ Q_B,Q_C},
		{ Q_B,Q_F},
		{ Q_C,Q_G},
		{ Q_C,Q_D},
		{ Q_D,Q_H},
		{ Q_E,Q_F},
		{ Q_E,Q_H},
		{ Q_H,Q_G},
		{ Q_G,Q_F}
	   };

void  LCD_Clear(unsigned int Von, unsigned int Bis)
{
unsigned int i,Wieviele;
 dput(Von%256);
 dput(Von>>8);
 cput(0x24);          // addrptr at address G_BASE
 Wieviele=Bis-Von+1;

 for (i=0;i!=Wieviele;i++)
  {
    dput(0);
    cput(0xC0);
  }
}

void G_Zeile(unsigned int Zeile)  // set the visible window to this line
{
if(Zeile>191) Zeile=127;
Zeile*=LCD_WIDTH/PIXPERBYTE;
Zeile+=G_BASE;

 dput(Zeile%256);
 dput(Zeile>>8);
 cput(0x42);       // set graphics memory to address G_BASE
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Shows a vector-object
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// W1,W2,W3 -> the rotation of the object around x-, y-, and z-axsis
// x_pos and Y_pos -> the Position in the Display-Ram 
// OBJ -> the Vector-Object to be shown
// cnt -> how many lines per Object
// size -> size: 128 is original size, 255 is double  
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RotatingObject(int W1,int W2,int W3, int x_Pos,int y_Pos,const int OBJ[][6],int cnt,int size)
{
 int M[4][4];   
 int g,i;
 int z1,z2 ;
 int x1,x2 ;
 int y1,y2 ;
 int xr1,xr2 ;
 int yr1,yr2 ;

  // Rotation-Matrix 
  // calculates the new position of the single vectors
  M[1][1] = ((long) COS[W2]*COS[W3]) /256;
  M[2][1] = ((long)-COS[W2]*SIN[W3]) /256;
  M[3][1] = SIN[W2];

  M[1][2] =  ((long)COS[W1]*SIN[W3])/256  + ((long)SIN[W1]*SIN[W2]*COS[W3])/((long)256 * 256);
  M[2][2] =  ((long)COS[W1]*COS[W3])/256  - ((long)SIN[W1]*SIN[W2]*SIN[W3])/((long)256 * 256);
  M[3][2] = ( (long) -SIN[W1]*COS[W2] ) / 256;

  M[1][3] =  ((long)SIN[W1]*SIN[W3])/256  - ((long)COS[W1]*SIN[W2]*COS[W3])/((long)256 * 256);
  M[2][3] =  ((long)SIN[W1]*COS[W3])/256  + ((long)COS[W1]*SIN[W2]*SIN[W3])/((long)256 * 256);

  M[3][3] =  ((long)COS[W1]*COS[W2])/256;

 if(size!=128)  // recalculate the size
  {
   for(g=1;g<4;g++) for(i=1;i<4;i++) M[g][i] =  (M[g][i] * size) / 128;
  };

 for(i=0;i<cnt;i++)  // draws the single lines
  {
    // get the line (from x1,y1,z1 to x2,y2,z2)
    x1 = OBJ[i][0];
    y1 = OBJ[i][1];					    
    z1 = OBJ[i][2];
    x2 = OBJ[i][3];
    y2 = OBJ[i][4];
    z2 = OBJ[i][5];

    // Convert the vector through the Rotation-Matrix
    xr1 = (M[1][1] * x1)/MUL + (M[2][1]*y1)/MUL + (M[3][1]*z1)/MUL;
    yr1 = (M[1][2] * x1)/MUL + (M[2][2]*y1)/MUL + (M[3][2]*z1)/MUL;
    xr2 = (M[1][1] * x2)/MUL + (M[2][1]*y2)/MUL + (M[3][1]*z2)/MUL;
    yr2 = (M[1][2] * x2)/MUL + (M[2][2]*y2)/MUL + (M[3][2]*z2)/MUL;

	// Draws the line
    LCD_Line( xr1 + x_Pos, yr1 + y_Pos, xr2 + x_Pos, yr2 + y_Pos);
   }
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// draws a circle by the simple equation : x = cos(i) and y = sin(i)
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void Circle(int x,int y,int radius)
{
int i;
  for(i=0; i<62;i++)  // 0 to 2*Pi
   { 
   lcd_setpixel( x + (COS[i] * radius) / 256 , y + (SIN[i] * radius) / 256);
   }
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// draws the Ball (planet?)
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void Ball(int W1,int W2,int W3, int x_Pos,int y_Pos,int size)
{
 int M[4][4];

register int g,i;
 int x,y,z;
 int xa,ya,za;
 int xb,yb;

  M[1][1] = ((long) COS[W2]*COS[W3]) /256;
  M[2][1] = ((long)-COS[W2]*SIN[W3]) /256;
  M[3][1] = SIN[W2];

  M[1][2] =  ((long)COS[W1]*SIN[W3])/256 + ((long)SIN[W1]*SIN[W2]*COS[W3])/((long)256 * 256);
  M[2][2] =  ((long)COS[W1]*COS[W3])/256 - ((long)SIN[W1]*SIN[W2]*SIN[W3])/((long)256 * 256);
  M[3][2] = ((long)-SIN[W1]*COS[W2])/256;

  M[1][3] =  ((long)SIN[W1]*SIN[W3])/256 - ((long)COS[W1]*SIN[W2]*COS[W3])/((long)256 * 256);
  M[2][3] =  ((long)SIN[W1]*COS[W3])/256 + ((long)COS[W1]*SIN[W2]*SIN[W3])/((long)256 * 256);
  M[3][3] =  ((long)COS[W1]*COS[W2])/256;

 if(size!=128)
  {
   for(g=1;g<4;g++) for(i=1;i<4;i++) M[g][i] =  (M[g][i] * size) / 128;
  };


 for (g=0;g<31;g+=8)
 {
   for (i=0;i<62;i++)
   {
    z = SIN[i] / 8;
    x = ((COS[i]/2) * (COS[g]/2)) / (2 * 256);
    y = ((COS[i]/2) * (SIN[g]/2)) / (2 * 256);

    ya = (M[1][2] * x)/MUL + (M[2][2]*y)/MUL + (M[3][2]*z)/MUL;
    if (ya >=0)	// just the positive y-values; otherwise the Ball is transparent
     {
       xa = (M[1][1] * x)/MUL + (M[2][1]*y)/MUL + (M[3][1]*z)/MUL;
       za = (M[1][3] * x)/MUL + (M[2][3]*y)/MUL + (M[3][3]*z)/MUL;
       xb = x_Pos + xa ;
       yb = y_Pos + za ;
	   lcd_setpixel(xb,yb);
     };
   }
 }
 for (i=15;i < 45;i+=4)  
 {
   for (g=0;g < 62; g++) 
   {
    z = SIN[i] / 8;
    x = ((COS[i]/2) * (COS[g]/2)) / (2 * 256);
    y = ((COS[i]/2) * (SIN[g]/2)) / (2 * 256);


    ya = (M[1][2] * x)/MUL + (M[2][2]*y)/MUL + (M[3][2]*z)/MUL;
     if (ya >=0) // just the positive y-values; otherwise the Ball is transparent
     {
       xa = (M[1][1] * x)/MUL + (M[2][1]*y)/MUL + (M[3][1]*z)/MUL;
       za = (M[1][3] * x)/MUL + (M[2][3]*y)/MUL + (M[3][3]*z)/MUL;
       xb = x_Pos + xa ;
       yb = y_Pos + za ;
	   lcd_setpixel(xb,yb);
     };
   }
 }
}


/**** MAIN **************************************************************/
 
int main(void)
{  
unsigned int Size = 0;
signed int p = 0;
char grows=1;
unsigned int Pos_x = 0;
int X_Rotation = 0, Y_Rotation = 0, Z_Rotation = 0;  

 InitDisplay();

 Cls();
 LCD_Clear(G_BASE,G_END1);  // clear the whole display

 cput(0x81);  // Graphic xor

// sprintf(Text,"H. Buss");
 LCD_Text(7,33,"H. Buss");

 while(1)  // always the same
 {
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Rotating pyramid
// The object is drawn in the not visible part of the Display-RAM
// After drawing this Part is shown 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  G_Zeile(0);  // set visible Display-position to line 20
// sprintf(Text,"Rotating objects");
 LCD_Text(0,1,"Rotating objects");
  for(p=0;p<100;p++)
   {
    X_Rotation+=1; // change the angles
    Y_Rotation+=1;
    Z_Rotation+=1;
    X_Rotation %= 62; Y_Rotation %= 62; Z_Rotation %= 62;  // because they should not become bigger than 62

    // draw the upper display-part
    LCD_Clear(G_BASE,G_END);
    RotatingObject(X_Rotation,Y_Rotation,Z_Rotation,  70,LCD_HEIGHT/2,PYR,CNT_PYR,128); // draw the object
    RotatingObject(X_Rotation,Y_Rotation,Z_Rotation,  30,LCD_HEIGHT/2,Q,CNT_Q,128); // draw the object
    RotatingObject(X_Rotation,Y_Rotation,Z_Rotation, 150,LCD_HEIGHT/2,UFO,CNT_UFO, 64); // draw the object
    G_Zeile(0);                    //show the upper display-part

    // draw the lower display-part
    X_Rotation+=1; // change the angles
    Z_Rotation+=1;
    LCD_Clear(G_BASE1,G_END1); // clear the not visible part of diplay-Ram
    RotatingObject(X_Rotation,Y_Rotation,Z_Rotation,  70,LCD_HEIGHT/2+LCD_HEIGHT,PYR,CNT_PYR,128); // draw the object
    RotatingObject(X_Rotation,Y_Rotation,Z_Rotation,  30,LCD_HEIGHT/2+LCD_HEIGHT,Q,CNT_Q,128); // draw the object
    RotatingObject(X_Rotation,Y_Rotation,Z_Rotation, 150,LCD_HEIGHT/2+LCD_HEIGHT,UFO,CNT_UFO, 64);  // draw the object
    G_Zeile(64);                   //show the lower display-part
   }

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// jumping Ball
// change the visible Display position so, that it looks like the Ball is jumping
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

 X_Rotation = 10;Y_Rotation = 10;Z_Rotation = 10;  
 LCD_Clear(G_BASE,G_END1);
// sprintf(Text,"Jumping Ball    ");
 LCD_Text(0,1,"Jumping Ball    ");

 Ball(X_Rotation,Y_Rotation,Z_Rotation, 128,LCD_HEIGHT/2,100); // x-,y-,z-Rotation,X-Position,Y-Position,Size
 for(p=0;p<64;p++)
  {
   G_Zeile(abs(SIN[(p + 32) % 64]) / 4); // change the visible Display 
   Delay1ms(200);
//   for(i=0;i<10000;i++);         // delay
  }

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Flying viper (Starship like in C64-Elite)
// The object is drawn in the not visible part of the Display-RAM
// After drawing this Part is shown 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  LCD_Clear(G_BASE,G_END);
//  sprintf(Text,"Flying viper    ");
  LCD_Text(0,1,"Flying viper    ");
  G_Zeile(64);
  Pos_x = 0;
  while(Pos_x < 500)
  {
   X_Rotation++; // new rotating variables
   Y_Rotation = 32;
   Z_Rotation = 0;
   X_Rotation %= 62;  Y_Rotation %= 62; 
   Pos_x += 2;	// fly from left to right

   // draw the upper display-part
   LCD_Clear(G_BASE,G_END); //clear the upper display-part
   RotatingObject(X_Rotation,Y_Rotation,Z_Rotation,Pos_x,LCD_HEIGHT/2,UFO,CNT_UFO, 64);
   G_Zeile(0);	// now the drawing is finished, so we can diplay this upper part

   X_Rotation+=1; // new rotating variables
   X_Rotation %= 62;
   Pos_x += 2;	// fly from left to right

   // draw the lower display-part
   LCD_Clear(G_BASE1,G_END1); // clear the not visible part of diplay-Ram
   RotatingObject(X_Rotation,Y_Rotation,Z_Rotation,Pos_x,LCD_HEIGHT/2+LCD_HEIGHT,UFO,CNT_UFO, 64);
   G_Zeile(64);                   //show the lower display-part

  }

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Viper flying around a Planet
// The object is also drawn in the not visible part of the Display-RAM
// After drawing this Part is shown 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

 X_Rotation = 0;Y_Rotation = 0;Z_Rotation = 0;Size = 0; grows = 1;
 for(p=0;p<100;p++)
  {
   X_Rotation+=1; // change the angles
   Y_Rotation+=1;
   Z_Rotation+=1;
   X_Rotation %= 62;    Y_Rotation %= 62;    Z_Rotation %= 62;

   // getting closer or farer
   if(grows)   Size+=4; 
   else	    Size-=4;
   if(Size == 0)   grows = 1;
   if(Size == 128) grows = 0;
  
   // draw the upper display-part
   LCD_Clear(G_BASE,G_END);
   Ball(X_Rotation,Y_Rotation,Z_Rotation, 128,LCD_HEIGHT/2,Size);
   RotatingObject(X_Rotation,Y_Rotation,Z_Rotation, 128 + (Size * SIN[X_Rotation])/256 ,LCD_HEIGHT/2,UFO,CNT_UFO, (Size * abs(SIN[X_Rotation/2]))/512);
   G_Zeile(0);

   // draw the lower display-part
   LCD_Clear(G_BASE1,G_END1);
   Ball(X_Rotation,Y_Rotation,Z_Rotation, 128,LCD_HEIGHT/2+LCD_HEIGHT,Size);
   RotatingObject(X_Rotation,Y_Rotation,Z_Rotation, 128 + (Size * SIN[X_Rotation])/256,LCD_HEIGHT/2+LCD_HEIGHT,UFO,CNT_UFO, (Size * abs(SIN[X_Rotation/2]))/512);
   G_Zeile(LCD_HEIGHT);
  }

 }//while(1);
}

//###################################################################################
//Quick hack stupid delay routines. Could be done better and shorter by using a timer
void Delay1ms(unsigned int time)
//###################################################################################
{
 unsigned int i;

 for(i=0; i<time; i++) Delay100us(10);
}

//###################################################################################
//Nicht sonderlich genau !!
//time<=255 !!
void Delay100us(unsigned char time)
//###################################################################################
{
 unsigned char k,l;

 for(k=0; k<time; k++)
  {
   for(l=0; l<40; l++)
    {
     //Ein CPU-Clock = 0.125us bei 8MHz
     NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();  
     NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();  
    }
  }
}


